@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
+ * Copyright (C) 2010-2011, 2013  Université de Bordeaux 1
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -34,8 +34,6 @@
 #include <string.h>
 #include <math.h>
 #include <sys/types.h>
-#include <sys/time.h>
-#include <pthread.h>
 #include <signal.h>
 
 #include <starpu.h>
@@ -78,7 +76,7 @@ static unsigned zdim = 512;
  * registered data with the "matrix" data interface, we use the matrix macros.
  */
 
-static void cpu_mult(void *descr[], __attribute__((unused))  void *arg)
+static void cpu_mult(void *descr[], STARPU_ATTRIBUTE_UNUSED  void *arg)
 {
 	float *subA, *subB, *subC;
 	uint32_t nxC, nyC, nyA;
@@ -194,13 +192,13 @@ static void partition_mult_data(void)
 	 * name of the filters are a bit misleading */
 	struct starpu_data_filter vert =
 	{
-		.filter_func = starpu_vertical_block_filter_func,
+		.filter_func = starpu_matrix_filter_vertical_block,
 		.nchildren = nslicesx
 	};
 
 	struct starpu_data_filter horiz =
 	{
-		.filter_func = starpu_block_filter_func,
+		.filter_func = starpu_matrix_filter_block,
 		.nchildren = nslicesy
 	};
 
@@ -263,7 +261,6 @@ static struct starpu_perfmodel mult_perf_model =
 static struct starpu_codelet cl =
 {
         /* we can only execute that kernel on a CPU yet */
-        .where = STARPU_CPU,
         /* CPU implementation of the codelet */
         .cpu_funcs = {cpu_mult, NULL},
         /* the codelet manipulates 3 buffers that are managed by the
@@ -336,8 +333,8 @@ static int launch_tasks(void)
 	return 0;
 }
 
-int main(__attribute__ ((unused)) int argc, 
-	 __attribute__ ((unused)) char **argv)
+int main(STARPU_ATTRIBUTE_UNUSED int argc, 
+	 STARPU_ATTRIBUTE_UNUSED char **argv)
 {
 	int ret;
 
