@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2013  Université de Bordeaux 1
+ * Copyright (C) 2010, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,11 +19,8 @@
 #define __PI_H__
 
 #include <starpu.h>
-#include <starpu_cuda.h>
 #include <stdio.h>
 
-#define NSHOT_PER_TASK	(16*1024*1024ULL)
-
 #define TYPE	float
 
 /* extern "C" void cuda_kernel(void *descr[], void *cl_arg); */
