@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2014  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014  Centre National de la Recherche Scientifique
  * Copyright (C) 2011  Télécom-SudParis
+ * Copyright (C) 2011  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,28 +28,7 @@
 #include <profiling/bound.h>
 #include <starpu_top.h>
 #include <top/starpu_top_core.h>
-
-size_t _starpu_job_get_data_size(struct starpu_perfmodel *model, enum starpu_perf_archtype arch, unsigned nimpl, struct _starpu_job *j)
-{
-	struct starpu_task *task = j->task;
-
-	if (model && model->per_arch[arch][nimpl].size_base) {
-		return model->per_arch[arch][nimpl].size_base(task, arch, nimpl);
-	} else if (model && model->size_base) {
-		return model->size_base(task, nimpl);
-	} else {
-		unsigned nbuffers = task->cl->nbuffers;
-		size_t size = 0;
-
-		unsigned buffer;
-		for (buffer = 0; buffer < nbuffers; buffer++)
-		{
-			starpu_data_handle_t handle = task->handles[buffer];
-			size += _starpu_data_get_size(handle);
-		}
-		return size;
-	}
-}
+#include <core/debug.h>
 
 /* we need to identify each task to generate the DAG. */
 static unsigned job_cnt = 0;
@@ -61,40 +41,45 @@ void _starpu_exclude_task_from_dag(struct starpu_task *task)
 }
 
 /* create an internal struct _starpu_job structure to encapsulate the task */
-struct _starpu_job* __attribute__((malloc)) _starpu_job_create(struct starpu_task *task)
+struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_task *task)
 {
 	struct _starpu_job *job;
         _STARPU_LOG_IN();
 
 	job = _starpu_job_new();
 
-	job->nimpl =0; /* best implementation */
+	/* As most of the fields must be initialized at NULL, let's put 0
+	 * everywhere */
+	memset(job, 0, sizeof(*job));
+
+	if (task->dyn_handles)
+	     job->dyn_ordered_buffers = malloc(task->cl->nbuffers * sizeof(job->dyn_ordered_buffers[0]));
+
 	job->task = task;
 
-	job->footprint_is_computed = 0;
-	job->submitted = 0;
-	job->terminated = 0;
-
 #ifndef STARPU_USE_FXT
-	if (_starpu_bound_recording || _starpu_top_status_get())
+	if (_starpu_bound_recording || _starpu_top_status_get()
+#ifdef HAVE_AYUDAME_H
+		|| AYU_event
 #endif
+			)
+#endif
+	{
 		job->job_id = STARPU_ATOMIC_ADD(&job_cnt, 1);
-#ifdef STARPU_USE_FXT
-	/* display all tasks by default */
-        job->model_name = NULL;
+#ifdef HAVE_AYUDAME_H
+		if (AYU_event)
+		{
+			/* Declare task to Ayudame */
+			int64_t AYU_data[2] = {_starpu_ayudame_get_func_id(task->cl), task->priority > STARPU_MIN_PRIO};
+			AYU_event(AYU_ADDTASK, job->job_id, AYU_data);
+		}
 #endif
-	job->exclude_from_dag = 0;
-
-	job->reduction_task = 0;
+	}
 
 	_starpu_cg_list_init(&job->job_successors);
 
-	job->implicit_dep_handle = NULL;
-
-	_STARPU_PTHREAD_MUTEX_INIT(&job->sync_mutex, NULL);
-	_STARPU_PTHREAD_COND_INIT(&job->sync_cond, NULL);
-
-	job->bound_task = NULL;
+	STARPU_PTHREAD_MUTEX_INIT(&job->sync_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT(&job->sync_cond, NULL);
 
 	/* By default we have sequential tasks */
 	job->task_size = 1;
@@ -110,18 +95,23 @@ void _starpu_job_destroy(struct _starpu_job *j)
 {
 	/* Wait for any code that was still working on the job (and was
 	 * probably our waker) */
-	_STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
-	_STARPU_PTHREAD_COND_DESTROY(&j->sync_cond);
-	_STARPU_PTHREAD_MUTEX_DESTROY(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+	STARPU_PTHREAD_COND_DESTROY(&j->sync_cond);
+	STARPU_PTHREAD_MUTEX_DESTROY(&j->sync_mutex);
 
 	if (j->task_size > 1)
 	{
-		_STARPU_PTHREAD_BARRIER_DESTROY(&j->before_work_barrier);
-		_STARPU_PTHREAD_BARRIER_DESTROY(&j->after_work_barrier);
+		STARPU_PTHREAD_BARRIER_DESTROY(&j->before_work_barrier);
+		STARPU_PTHREAD_BARRIER_DESTROY(&j->after_work_barrier);
 	}
 
 	_starpu_cg_list_deinit(&j->job_successors);
+	if (j->dyn_ordered_buffers)
+	{
+	     free(j->dyn_ordered_buffers);
+	     j->dyn_ordered_buffers = NULL;
+	}
 
 	_starpu_job_delete(j);
 }
@@ -132,7 +122,7 @@ void _starpu_wait_job(struct _starpu_job *j)
 	STARPU_ASSERT(!j->task->detach);
         _STARPU_LOG_IN();
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 
 	/* We wait for the flag to have a value of 2 which means that both the
 	 * codelet's implementation and its callback have been executed. That
@@ -140,17 +130,18 @@ void _starpu_wait_job(struct _starpu_job *j)
 	 * executed (so that we cannot destroy the task while it is still being
 	 * manipulated by the driver). */
 	while (j->terminated != 2)
-		_STARPU_PTHREAD_COND_WAIT(&j->sync_cond, &j->sync_mutex);
+		STARPU_PTHREAD_COND_WAIT(&j->sync_cond, &j->sync_mutex);
 
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
         _STARPU_LOG_OUT();
 }
 
 void _starpu_handle_job_termination(struct _starpu_job *j)
 {
 	struct starpu_task *task = j->task;
-
-	_STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	unsigned sched_ctx = task->sched_ctx;
+	double flops = task->flops;
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 
 	task->status = STARPU_TASK_FINISHED;
 
@@ -160,13 +151,51 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	 * the callback is not done yet. */
 	j->terminated = 1;
 
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
+#ifdef STARPU_USE_SC_HYPERVISOR
+	size_t data_size = 0;
+	int workerid = starpu_worker_get_id();
+#endif //STARPU_USE_SC_HYPERVISOR
+
+	/* We release handle reference count */
+	if (task->cl)
+	{
+		unsigned i;
+#ifdef STARPU_USE_SC_HYPERVISOR
+		for(i = 0; i < task->cl->nbuffers; i++)
+		{
+			starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, i);
+			if (handle != NULL)
+				data_size += _starpu_data_get_size(handle);
+		}
+#endif //STARPU_USE_SC_HYPERVISOR
+
+		for (i = 0; i < task->cl->nbuffers; i++)
+		{
+			starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, i);
+			_starpu_spin_lock(&handle->header_lock);
+			handle->busy_count--;
+			if (!_starpu_data_check_not_busy(handle))
+				_starpu_spin_unlock(&handle->header_lock);
+		}
+	}
+
+	/* Tell other tasks that we don't exist any more, thus no need for
+	 * implicit dependencies any more.  */
+	_starpu_release_task_enforce_sequential_consistency(j);
 	/* Task does not have a cl, but has explicit data dependencies, we need
 	 * to tell them that we will not exist any more before notifying the
 	 * tasks waiting for us */
-	if (j->implicit_dep_handle)
-		_starpu_release_data_enforce_sequential_consistency(j->task, j->implicit_dep_handle);
+	if (j->implicit_dep_handle) {
+		starpu_data_handle_t handle = j->implicit_dep_handle;
+		_starpu_release_data_enforce_sequential_consistency(j->task, handle);
+		/* Release reference taken while setting implicit_dep_handle */
+		_starpu_spin_lock(&handle->header_lock);
+		handle->busy_count--;
+		if (!_starpu_data_check_not_busy(handle))
+			_starpu_spin_unlock(&handle->header_lock);
+	}
 
 	/* in case there are dependencies, wake up the proper tasks */
 	_starpu_notify_dependencies(j);
@@ -206,8 +235,15 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	/* If the job was executed on a combined worker there is no need for the
 	 * scheduler to process it : the task structure doesn't contain any valuable
 	 * data as it's not linked to an actual worker */
-	if (j->task_size == 1)
+	/* control task should not execute post_exec_hook */
+	if(j->task_size == 1 && task->cl != NULL && !j->internal)
+	{
 		_starpu_sched_post_exec_hook(task);
+#ifdef STARPU_USE_SC_HYPERVISOR
+		_starpu_sched_ctx_post_exec_task_cb(workerid, task, data_size, j->footprint);
+#endif //STARPU_USE_SC_HYPERVISOR
+
+	}
 
 	_STARPU_TRACE_TASK_DONE(j);
 
@@ -220,13 +256,17 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 
 	/* we do not desallocate the job structure if some is going to
 	 * wait after the task */
-	_STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 	/* A value of 2 is put to specify that not only the codelet but
 	 * also the callback were executed. */
 	j->terminated = 2;
-	_STARPU_PTHREAD_COND_BROADCAST(&j->sync_cond);
+	STARPU_PTHREAD_COND_BROADCAST(&j->sync_cond);
 
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+#ifdef HAVE_AYUDAME_H
+	if (AYU_event) AYU_event(AYU_REMOVETASK, j->job_id, NULL);
+#endif
+
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
 	if (detach)
 	{
@@ -240,14 +280,36 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 
 	if (regenerate)
 	{
-		STARPU_ASSERT_MSG(detach && !destroy && !task->synchronous, "Regenerated task must be detached, and not have detroy=1 or synchronous=1");
+		STARPU_ASSERT_MSG(detach && !destroy && !task->synchronous, "Regenerated task must be detached (was %d), and not have detroy=1 (was %d) or synchronous=1 (was %d)", detach, destroy, task->synchronous);
+
+#ifdef HAVE_AYUDAME_H
+		if (AYU_event)
+		{
+			int64_t AYU_data[2] = {j->exclude_from_dag?0:_starpu_ayudame_get_func_id(task->cl), task->priority > STARPU_MIN_PRIO};
+			AYU_event(AYU_ADDTASK, j->job_id, AYU_data);
+		}
+#endif
 
 		/* We reuse the same job structure */
 		int ret = _starpu_submit_job(j);
 		STARPU_ASSERT(!ret);
 	}
-	_starpu_decrement_nsubmitted_tasks();
-	_starpu_decrement_nready_tasks();
+
+	_starpu_decrement_nsubmitted_tasks_of_sched_ctx(sched_ctx);
+	_starpu_decrement_nready_tasks_of_sched_ctx(sched_ctx, flops);
+	struct _starpu_worker *worker;
+	worker = _starpu_get_local_worker_key();
+	if (worker)
+	{
+		STARPU_PTHREAD_MUTEX_LOCK(&worker->sched_mutex);
+
+		if(worker->removed_from_ctx[sched_ctx] == 1 && worker->shares_tasks_lists[sched_ctx] == 1)
+		{
+			_starpu_worker_gets_out_of_ctx(sched_ctx, worker);
+			worker->removed_from_ctx[sched_ctx] = 0;
+		}
+		STARPU_PTHREAD_MUTEX_UNLOCK(&worker->sched_mutex);
+	}
 }
 
 /* This function is called when a new task is submitted to StarPU
@@ -267,6 +329,7 @@ static unsigned _starpu_not_all_tag_deps_are_fulfilled(struct _starpu_job *j)
 	struct _starpu_cg_list *tag_successors = &tag->tag_successors;
 
 	_starpu_spin_lock(&tag->lock);
+	STARPU_ASSERT_MSG(tag->is_assigned == 1 || !tag_successors->ndeps, "a tag can be assigned only one task to wake (%llu had %u assigned tasks, and %u successors)", (unsigned long long) tag->id, tag->is_assigned, tag_successors->ndeps);
 
 	if (tag_successors->ndeps != tag_successors->ndeps_completed)
 	{
@@ -328,30 +391,30 @@ unsigned _starpu_enforce_deps_and_schedule(struct _starpu_job *j)
 	/* enfore tag dependencies */
 	if (_starpu_not_all_tag_deps_are_fulfilled(j))
 	{
-		_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
-                _STARPU_LOG_OUT_TAG("not_all_tag_deps_are_fulfilled");
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		_STARPU_LOG_OUT_TAG("not_all_tag_deps_are_fulfilled");
 		return 0;
-        }
+	}
 
 	/* enfore task dependencies */
 	if (_starpu_not_all_task_deps_are_fulfilled(j))
 	{
-		_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
-                _STARPU_LOG_OUT_TAG("not_all_task_deps_are_fulfilled");
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		_STARPU_LOG_OUT_TAG("not_all_task_deps_are_fulfilled");
 		return 0;
-        }
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+	}
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
 	/* enforce data dependencies */
 	if (_starpu_submit_job_enforce_data_deps(j))
 	{
-                _STARPU_LOG_OUT_TAG("enforce_data_deps");
+		_STARPU_LOG_OUT_TAG("enforce_data_deps");
 		return 0;
-        }
+	}
 
 	ret = _starpu_push_task(j);
 
-        _STARPU_LOG_OUT();
+	_STARPU_LOG_OUT();
 	return ret;
 }
 
@@ -360,14 +423,13 @@ unsigned _starpu_enforce_deps_starting_from_task(struct _starpu_job *j)
 {
 	unsigned ret;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 	/* enfore task dependencies */
 	if (_starpu_not_all_task_deps_are_fulfilled(j))
 	{
-		_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 		return 0;
 	}
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
 	/* enforce data dependencies */
 	if (_starpu_submit_job_enforce_data_deps(j))
@@ -384,48 +446,28 @@ struct starpu_task *_starpu_pop_local_task(struct _starpu_worker *worker)
 	struct starpu_task *task = NULL;
 
 	if (!starpu_task_list_empty(&worker->local_tasks))
-		task = starpu_task_list_pop_back(&worker->local_tasks);
+		task = starpu_task_list_pop_front(&worker->local_tasks);
 
 	return task;
 }
 
-int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task, int back)
+int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task, int prio)
 {
 	/* Check that the worker is able to execute the task ! */
 	STARPU_ASSERT(task && task->cl);
 	if (STARPU_UNLIKELY(!(worker->worker_mask & task->cl->where)))
 		return -ENODEV;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(worker->sched_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&worker->sched_mutex);
 
-	if (back)
-		starpu_task_list_push_back(&worker->local_tasks, task);
-	else
+	if (prio)
 		starpu_task_list_push_front(&worker->local_tasks, task);
+	else
+		starpu_task_list_push_back(&worker->local_tasks, task);
 
-	_STARPU_PTHREAD_COND_BROADCAST(worker->sched_cond);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(worker->sched_mutex);
+	STARPU_PTHREAD_COND_BROADCAST(&worker->sched_cond);
+	starpu_push_task_end(task);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&worker->sched_mutex);
 
 	return 0;
 }
-
-const char *_starpu_get_model_name(struct _starpu_job *j)
-{
-	if (!j)
-		return NULL;
-
-	struct starpu_task *task = j->task;
-        if (task && task->cl) {
-            if (task->cl->model && task->cl->model->symbol)
-                return task->cl->model->symbol;
-	    else
-		return task->cl->name;
-	} else
-	{
-#ifdef STARPU_USE_FXT
-                return j->model_name;
-#else
-                return NULL;
-#endif
-        }
-}
