@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2014  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -30,6 +30,7 @@
 #include <datawizard/data_request.h>
 #include <datawizard/interfaces/data_interface.h>
 #include <datawizard/datastats.h>
+#include <datawizard/memstats.h>
 
 enum _starpu_cache_state
 {
@@ -42,32 +43,31 @@ enum _starpu_cache_state
 LIST_TYPE(_starpu_data_replicate,
 	starpu_data_handle_t handle;
 
-	/* describe the actual data layout */
+	/* describe the actual data layout, as manipulated by data interfaces in *_interface.c */
 	void *data_interface;
 
 	unsigned memory_node;
 
-	/* A buffer that is used for SCRATCH or reduction cannnot be used with
-	 * filters. */
-	unsigned relaxed_coherency;
-
-	/* In the case of a SCRATCH access, we need to initialize the replicate
-	 * with a neutral element before using it. */
-	unsigned initialized;
-
 	/* describes the state of the local data in term of coherency */
 	enum _starpu_cache_state	state;
 
 	int refcnt;
 
+	/* A buffer that is used for SCRATCH or reduction cannnot be used with
+	 * filters. */
+	unsigned relaxed_coherency:2;
+
+	/* We may need to initialize the replicate with some value before using it. */
+	unsigned initialized:1;
+
 	/* is the data locally allocated ? */
-	uint8_t allocated;
+	unsigned allocated:1;
 	/* was it automatically allocated ? (else it's the application-provided
 	 * buffer, don't ever try to free it!) */
 	/* perhaps the allocation was perform higher in the hiearchy
 	 * for now this is just translated into !automatically_allocated
 	 * */
-	uint8_t automatically_allocated;
+	unsigned automatically_allocated:1;
 
         /* Pointer to memchunk for LRU strategy */
 	struct _starpu_mem_chunk * mc;
@@ -79,7 +79,7 @@ LIST_TYPE(_starpu_data_replicate,
 	   flag when it assigns a task to a queue, policies which do not
 	   use this hint can simply ignore it.
 	 */
-	uint8_t requested[STARPU_MAXNODES];
+	uint32_t requested;
 	struct _starpu_data_request *request[STARPU_MAXNODES];
 )
 
@@ -98,6 +98,10 @@ struct _starpu_task_wrapper_list
 	struct _starpu_task_wrapper_list *next;
 };
 
+extern int _starpu_has_not_important_data;
+
+typedef void (*_starpu_data_handle_unregister_hook)(starpu_data_handle_t);
+
 struct _starpu_data_state
 {
 	struct _starpu_data_requester_list *req_list;
@@ -105,19 +109,19 @@ struct _starpu_data_state
 	 * the req_list anymore), i.e. the number of holders of the
 	 * current_mode rwlock */
 	unsigned refcnt;
-	enum starpu_access_mode current_mode;
+	enum starpu_data_access_mode current_mode;
 	/* protect meta data */
 	struct _starpu_spinlock header_lock;
 
 	/* Condition to make application wait for all transfers before freeing handle */
-	/* busy_count is the number of handle->refcnt, handle->per_node[*]->refcnt, and number of starpu_data_requesters */
+	/* busy_count is the number of handle->refcnt, handle->per_node[*]->refcnt, number of starpu_data_requesters, and number of tasks that have released it but are still registered on the implicit data dependency lists. */
 	/* Core code which releases busy_count has to call
 	 * _starpu_data_check_not_busy to let starpu_data_unregister proceed */
 	unsigned busy_count;
 	/* Is starpu_data_unregister waiting for busy_count? */
 	unsigned busy_waiting;
-	pthread_mutex_t busy_mutex;
-	pthread_cond_t busy_cond;
+	starpu_pthread_mutex_t busy_mutex;
+	starpu_pthread_cond_t busy_cond;
 
 	/* In case we user filters, the handle may describe a sub-data */
 	struct _starpu_data_state *root_handle; /* root of the tree */
@@ -125,7 +129,7 @@ struct _starpu_data_state
 	unsigned sibling_index; /* indicate which child this node is from the father's perpsective (if any) */
 	unsigned depth; /* what's the depth of the tree ? */
 
-	struct _starpu_data_state *children;
+	starpu_data_handle_t children;
 	unsigned nchildren;
 
 	/* describe the state of the data in term of coherency */
@@ -134,9 +138,6 @@ struct _starpu_data_state
 
 	struct starpu_data_interface_ops *ops;
 
-	/* To avoid recomputing data size all the time, we store it directly. */
-	size_t data_size;
-
 	/* Footprint which identifies data layout */
 	uint32_t footprint;
 
@@ -158,13 +159,13 @@ struct _starpu_data_state
 
 	/* This lock should protect any operation to enforce
 	 * sequential_consistency */
-	pthread_mutex_t sequential_consistency_mutex;
+	starpu_pthread_mutex_t sequential_consistency_mutex;
 
 	/* The last submitted task (or application data request) that declared
 	 * it would modify the piece of data ? Any task accessing the data in a
 	 * read-only mode should depend on that task implicitely if the
 	 * sequential_consistency flag is enabled. */
-	enum starpu_access_mode last_submitted_mode;
+	enum starpu_data_access_mode last_submitted_mode;
 	struct starpu_task *last_submitted_writer;
 	struct _starpu_task_wrapper_list *last_submitted_readers;
 
@@ -178,6 +179,7 @@ struct _starpu_data_state
 	unsigned long last_submitted_ghost_writer_id;
 	struct _starpu_jobid_list *last_submitted_ghost_readers_id;
 
+	/* protected by sequential_consistency_mutex */
 	struct _starpu_task_wrapper_list *post_sync_tasks;
 	unsigned post_sync_tasks_cnt;
 
@@ -202,7 +204,7 @@ struct _starpu_data_state
 	 * the end of the reduction. */
 	struct _starpu_data_requester_list *reduction_req_list;
 
-	starpu_data_handle_t reduction_tmp_handles[STARPU_NMAXWORKERS];
+	starpu_data_handle_t *reduction_tmp_handles;
 
 	unsigned lazy_unregister;
 
@@ -210,16 +212,12 @@ struct _starpu_data_state
         int rank;
 	int tag;
 
-#ifdef STARPU_MEMORY_STATUS
-	/* Handle access stats per node */
-	unsigned stats_direct_access[STARPU_MAXNODES];
-	unsigned stats_loaded_shared[STARPU_MAXNODES];
-	unsigned stats_loaded_owner[STARPU_MAXNODES];
-	unsigned stats_shared_to_owner[STARPU_MAXNODES];
-	unsigned stats_invalidated[STARPU_MAXNODES];
-#endif
+	_starpu_memory_stats_t memory_stats;
 
 	unsigned int mf_node; //XXX
+
+	/* hook to be called when unregistering the data */
+	_starpu_data_handle_unregister_hook unregister_hook;
 };
 
 void _starpu_display_msi_stats(void);
@@ -231,7 +229,7 @@ void _starpu_display_msi_stats(void);
  * async means that _starpu_fetch_data_on_node will wait for completion of the request
  */
 int _starpu_fetch_data_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate,
-			       enum starpu_access_mode mode, unsigned detached, unsigned async,
+			       enum starpu_data_access_mode mode, unsigned detached, unsigned async,
 			       void (*callback_func)(void *), void *callback_arg);
 /* This releases a reference on the handle */
 void _starpu_release_data_on_node(struct _starpu_data_state *state, uint32_t default_wt_mask,
@@ -239,9 +237,9 @@ void _starpu_release_data_on_node(struct _starpu_data_state *state, uint32_t def
 
 void _starpu_update_data_state(starpu_data_handle_t handle,
 			       struct _starpu_data_replicate *requesting_replicate,
-			       enum starpu_access_mode mode);
+			       enum starpu_data_access_mode mode);
 
-uint32_t _starpu_get_data_refcnt(struct _starpu_data_state *state, uint32_t node);
+uint32_t _starpu_get_data_refcnt(struct _starpu_data_state *state, unsigned node);
 
 size_t _starpu_data_get_size(starpu_data_handle_t handle);
 
@@ -249,14 +247,12 @@ uint32_t _starpu_data_get_footprint(starpu_data_handle_t handle);
 
 void _starpu_push_task_output(struct _starpu_job *j, uint32_t mask);
 
-__attribute__((warn_unused_result))
+STARPU_ATTRIBUTE_WARN_UNUSED_RESULT
 int _starpu_fetch_task_input(struct _starpu_job *j, uint32_t mask);
 
-unsigned _starpu_is_data_present_or_requested(struct _starpu_data_state *state, uint32_t node);
-unsigned starpu_data_test_if_allocated_on_node(starpu_data_handle_t handle, uint32_t memory_node);
+unsigned _starpu_is_data_present_or_requested(struct _starpu_data_state *state, unsigned node);
 
-
-uint32_t _starpu_select_src_node(struct _starpu_data_state *state, unsigned destination);
+int _starpu_select_src_node(struct _starpu_data_state *state, unsigned destination);
 
 /* is_prefetch is whether the DSM may drop the request (when there is not enough memory for instance
  * async is whether the caller wants a reference on the last request, to be
@@ -264,7 +260,7 @@ uint32_t _starpu_select_src_node(struct _starpu_data_state *state, unsigned dest
  */
 struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_handle_t handle,
 								  struct _starpu_data_replicate *dst_replicate,
-								  enum starpu_access_mode mode, unsigned is_prefetch,
+								  enum starpu_data_access_mode mode, unsigned is_prefetch,
 								  unsigned async,
 								  void (*callback_func)(void *), void *callback_arg);
 
@@ -273,4 +269,6 @@ void _starpu_data_start_reduction_mode(starpu_data_handle_t handle);
 void _starpu_data_end_reduction_mode(starpu_data_handle_t handle);
 void _starpu_data_end_reduction_mode_terminate(starpu_data_handle_t handle);
 
+void _starpu_data_set_unregister_hook(starpu_data_handle_t handle, _starpu_data_handle_unregister_hook func);
+
 #endif // __COHERENCY__H__
