@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2012-2014  Université de Bordeaux 1
+ * Copyright (C) 2010, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,10 +40,23 @@
 
 unsigned _starpu_get_cuda_device_count(void);
 
-#ifdef STARPU_USE_CUDA
+#if defined(STARPU_USE_CUDA) || defined(STARPU_SIMGRID)
+void _starpu_cuda_discover_devices (struct _starpu_machine_config *);
 void _starpu_init_cuda(void);
 void *_starpu_cuda_worker(void *);
-cudaStream_t starpu_cuda_get_local_transfer_stream(void);
+#else
+#  define _starpu_cuda_discover_devices(config) ((void) config)
+#endif
+#ifdef STARPU_USE_CUDA
+cudaStream_t starpu_cuda_get_local_in_transfer_stream(void);
+cudaStream_t starpu_cuda_get_local_out_transfer_stream(void);
+cudaStream_t starpu_cuda_get_peer_transfer_stream(unsigned src_node, unsigned dst_node);
+
+struct _starpu_worker;
+int _starpu_run_cuda(struct _starpu_worker *);
+int _starpu_cuda_driver_init(struct _starpu_worker *);
+int _starpu_cuda_driver_run_once(struct _starpu_worker *);
+int _starpu_cuda_driver_deinit(struct _starpu_worker *);
 #endif
 
 #endif //  __DRIVER_CUDA_H__
