@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010, 2012  Université de Bordeaux 1
+ * Copyright (C) 2012, 2013  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,31 +15,27 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+#include <config.h>
 #include <starpu.h>
-#ifdef STARPU_USE_CUDA
-#include <cuda.h>
-#endif
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
+#include "../helper.h"
 
 #define INIT_VALUE	42
 #define NTASKS		10000
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
 
 static unsigned variable;
-static starpu_data_handle variable_handle;
+static starpu_data_handle_t variable_handle;
 
 static uintptr_t per_worker[STARPU_NMAXWORKERS];
-static starpu_data_handle per_worker_handle[STARPU_NMAXWORKERS];
+static starpu_data_handle_t per_worker_handle[STARPU_NMAXWORKERS];
 
 /* Create per-worker handles */
-static void initialize_per_worker_handle(void *arg __attribute__((unused)))
+static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 	int workerid = starpu_worker_get_id();
-	
+
 	/* Allocate memory on the worker, and initialize it to 0 */
-	switch (starpu_worker_get_type(workerid)) {
+	switch (starpu_worker_get_type(workerid))
+	{
 		case STARPU_CPU_WORKER:
 			per_worker[workerid] = (uintptr_t)calloc(1, sizeof(variable));
 			break;
@@ -53,10 +50,11 @@ static void initialize_per_worker_handle(void *arg __attribute__((unused)))
 			cl_mem ptr = clCreateBuffer(context, CL_MEM_READ_WRITE, sizeof(variable), NULL, NULL);
 			/* Poor's man memset */
 			unsigned zero = 0;
-			clEnqueueWriteBuffer(queue, ptr, CL_TRUE, 0, sizeof(variable), (void *)&zero, 0, NULL, NULL); 
+			clEnqueueWriteBuffer(queue, ptr, CL_FALSE, 0, sizeof(variable), (void *)&zero, 0, NULL, NULL);
+			clFinish(queue);
 			per_worker[workerid] = (uintptr_t)ptr;
 			}
-			
+
 			break;
 #endif
 #ifdef STARPU_USE_CUDA
@@ -77,7 +75,7 @@ static void initialize_per_worker_handle(void *arg __attribute__((unused)))
  *	Implement reduction method
  */
 
-static void cpu_redux_func(void *descr[], void *cl_arg __attribute__((unused)))
+static void cpu_redux_func(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
 	unsigned *a = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned *b = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[1]);
@@ -87,11 +85,12 @@ static void cpu_redux_func(void *descr[], void *cl_arg __attribute__((unused)))
 	*a = *a + *b;
 }
 
-static struct starpu_codelet_t reduction_codelet = {
-	.where = STARPU_CPU,
-	.cpu_func = cpu_redux_func,
-	.cuda_func = NULL,
+static struct starpu_codelet reduction_codelet =
+{
+	.cpu_funcs = {cpu_redux_func, NULL},
+	.cuda_funcs = {NULL},
 	.nbuffers = 2,
+	.modes = {STARPU_RW, STARPU_R},
 	.model = NULL
 };
 
@@ -99,51 +98,60 @@ static struct starpu_codelet_t reduction_codelet = {
  *	Use per-worker local copy
  */
 
-static void cpu_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+static void cpu_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*val = *val + 1;
 }
 
-#ifdef STARPU_USE_CUDA 
+#ifdef STARPU_USE_CUDA
 /* dummy CUDA implementation */
-static void cuda_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+static void cuda_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
+	STARPU_SKIP_IF_VALGRIND;
+
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	unsigned h_val;
-	cudaMemcpy(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost);
+	cudaMemcpyAsync(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 	h_val++;
-	cudaMemcpy(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice);
+	cudaMemcpyAsync(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
+	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
 #endif
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void opencl_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+static void opencl_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
+	STARPU_SKIP_IF_VALGRIND;
+
 	cl_mem d_val = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned h_val;
 
 	cl_command_queue queue;
 	starpu_opencl_get_current_queue(&queue);
 
-	clEnqueueReadBuffer(queue, d_val, CL_TRUE, 0, sizeof(unsigned), (void *)&h_val, 0, NULL, NULL);
+	clEnqueueReadBuffer(queue, d_val, CL_FALSE, 0, sizeof(unsigned), (void *)&h_val, 0, NULL, NULL);
+	clFinish(queue);
 	h_val++;
-	clEnqueueWriteBuffer(queue, d_val, CL_TRUE, 0, sizeof(unsigned), (void *)&h_val, 0, NULL, NULL); 
+	clEnqueueWriteBuffer(queue, d_val, CL_FALSE, 0, sizeof(unsigned), (void *)&h_val, 0, NULL, NULL);
+	clFinish(queue);
 }
 #endif
 
-static struct starpu_codelet_t use_data_on_worker_codelet = {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_func = cpu_func_incr,
+static struct starpu_codelet use_data_on_worker_codelet =
+{
+	.cpu_funcs = {cpu_func_incr, NULL},
 #ifdef STARPU_USE_CUDA
-	.cuda_func = cuda_func_incr,
+	.cuda_funcs = {cuda_func_incr, NULL},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_func = opencl_func_incr,
+	.opencl_funcs = {opencl_func_incr, NULL},
 #endif
 	.nbuffers = 1,
+	.modes = {STARPU_RW},
 	.model = NULL
 };
 
@@ -151,10 +159,13 @@ int main(int argc, char **argv)
 {
 	unsigned worker;
 	unsigned i;
+	int ret;
 
 	variable = INIT_VALUE;
 
-        starpu_init(NULL);
+        ret = starpu_init(NULL);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	unsigned nworkers = starpu_worker_get_count();
 
@@ -180,15 +191,14 @@ int main(int argc, char **argv)
 		task->cl = &use_data_on_worker_codelet;
 
 		int workerid = (i % nworkers);
-		task->buffers[0].handle = per_worker_handle[workerid];
-		task->buffers[0].mode = STARPU_RW;
+		task->handles[0] = per_worker_handle[workerid];
 
 		task->execute_on_a_specific_worker = 1;
 		task->workerid = (unsigned)workerid;
 
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
-		STARPU_ASSERT(!ret);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
 
@@ -198,32 +208,53 @@ int main(int argc, char **argv)
 		struct starpu_task *task = starpu_task_create();
 		task->cl = &reduction_codelet;
 
-		task->buffers[0].handle = variable_handle;
-		task->buffers[0].mode = STARPU_RW;
+		task->handles[0] = variable_handle;
+		task->handles[1] = per_worker_handle[worker];
 
-		task->buffers[1].handle = per_worker_handle[worker];
-		task->buffers[1].mode = STARPU_R;
-
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
-		STARPU_ASSERT(!ret);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
 	starpu_data_unregister(variable_handle);
 
 	/* Destroy all per-worker handles */
 	for (worker = 0; worker < nworkers; worker++)
+	{
 		starpu_data_unregister_no_coherency(per_worker_handle[worker]);
-
-	STARPU_ASSERT(variable == (INIT_VALUE + NTASKS));
+		switch(starpu_worker_get_type(worker))
+		{
+		case STARPU_CPU_WORKER:
+			free((void*)per_worker[worker]);
+			break;
+#ifdef STARPU_USE_CUDA
+		case STARPU_CUDA_WORKER:
+			cudaFree((void*)per_worker[worker]);
+			break;
+#endif /* !STARPU_USE_CUDA */
+#ifdef STARPU_USE_OPENCL
+		case STARPU_OPENCL_WORKER:
+			clReleaseMemObject((void*)per_worker[worker]);
+			break;
+#endif /* !STARPU_USE_OPENCL */
+		default:
+			STARPU_ABORT();
+		}
+	}
 
 	starpu_shutdown();
 
-	return 0;
+	if (variable == INIT_VALUE + NTASKS)
+		ret = EXIT_SUCCESS;
+	else
+		ret = EXIT_FAILURE;
+	STARPU_RETURN(ret);
 
 enodev:
 	fprintf(stderr, "WARNING: No one can execute this task\n");
+	starpu_task_wait_for_all();
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
-	return 77;
+	starpu_shutdown();
+	return STARPU_TEST_SKIPPED;
 }
