@@ -27,7 +27,6 @@ static void dummy_kernel(void *descr[], void *arg)
 
 static struct starpu_codelet r_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 	.cuda_funcs = {dummy_kernel, NULL},
 	.cpu_funcs = {dummy_kernel, NULL},
 	.opencl_funcs = {dummy_kernel, NULL},
@@ -37,7 +36,6 @@ static struct starpu_codelet r_cl =
 
 static struct starpu_codelet w_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 	.cuda_funcs = {dummy_kernel, NULL},
 	.cpu_funcs = {dummy_kernel, NULL},
 	.opencl_funcs = {dummy_kernel, NULL},
@@ -71,13 +69,13 @@ int main(int argc, char **argv)
 
 		/* we randomly select either a reader or a writer (give 10
 		 * times more chances to be a reader) */
-		enum starpu_access_mode mode = ((rand() % 10)==0)?STARPU_W:STARPU_R;
+		enum starpu_data_access_mode mode = ((rand() % 10)==0)?STARPU_W:STARPU_R;
 		if (mode == STARPU_W)
 			task->cl = &w_cl;
 		else
 			task->cl = &r_cl;
 
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
