@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2014  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +15,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <sys/time.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
@@ -30,8 +29,8 @@
   #define N	10
 #endif
 
-struct timeval start;
-struct timeval end;
+static double start;
+static double end;
 
 int main(int argc, char **argv)
 {
@@ -43,20 +42,20 @@ int main(int argc, char **argv)
 
 	for (iter = 0; iter < N; iter++)
 	{
-		gettimeofday(&start, NULL);
+		start = starpu_timing_now();
 		/* Initialize StarPU */
 		ret = starpu_init(NULL);
-		gettimeofday(&end, NULL);
+		end = starpu_timing_now();
 		if (ret == -ENODEV)
 			goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-		init_timing += (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
+		init_timing += end - start;
 
-		gettimeofday(&start, NULL);
+		start = starpu_timing_now();
 		/* Shutdown StarPU */
 		starpu_shutdown();
-		gettimeofday(&end, NULL);
-		shutdown_timing += (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
+		end = starpu_timing_now();
+		shutdown_timing += end - start;
 	}
 
 	FPRINTF(stderr, "starpu_init: %2.2f seconds\n", init_timing/(N*1000000));
