@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2011, 2013-2014  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,10 +15,8 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <sys/time.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <pthread.h>
 
 #include <starpu.h>
 #include "../helper.h"
@@ -26,18 +24,21 @@
 starpu_data_handle_t data_handles[8];
 float *buffers[8];
 
+#ifdef STARPU_QUICK_CHECK
+static unsigned ntasks = 128;
+#else
 static unsigned ntasks = 65536;
+#endif
 static unsigned nbuffers = 0;
 
 struct starpu_task *tasks;
 
-static void dummy_func(void *descr[] __attribute__ ((unused)), void *arg __attribute__ ((unused)))
+static void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 }
 
-static struct starpu_codelet dummy_codelet = 
+static struct starpu_codelet dummy_codelet =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 	.cpu_funcs = {dummy_func, NULL},
 	.cuda_funcs = {dummy_func, NULL},
 	.opencl_funcs = {dummy_func, NULL},
@@ -85,12 +86,12 @@ int main(int argc, char **argv)
 	unsigned i;
 
 	double timing_submit;
-	struct timeval start_submit;
-	struct timeval end_submit;
+	double start_submit;
+	double end_submit;
 
 	double timing_exec;
-	struct timeval start_exec;
-	struct timeval end_exec;
+	double start_exec;
+	double end_exec;
 
 	parse_args(argc, argv);
 
@@ -110,7 +111,7 @@ int main(int argc, char **argv)
 	/* submit tasks (but don't execute them yet !) */
 	tasks = (struct starpu_task *) calloc(1, ntasks*sizeof(struct starpu_task));
 
-	gettimeofday(&start_submit, NULL);
+	start_submit = starpu_timing_now();
 	for (i = 0; i < ntasks; i++)
 	{
 		starpu_task_init(&tasks[i]);
@@ -127,8 +128,9 @@ int main(int argc, char **argv)
 			tasks[i].handles[buffer] = data_handles[buffer];
 		}
 	}
+	tasks[ntasks-1].detach = 0;
 
-	gettimeofday(&start_submit, NULL);
+	start_submit = starpu_timing_now();
 	for (i = 1; i < ntasks; i++)
 	{
 		starpu_tag_declare_deps((starpu_tag_t)i, 1, (starpu_tag_t)(i-1));
@@ -143,21 +145,24 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) goto enodev;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
-	gettimeofday(&end_submit, NULL);
+	end_submit = starpu_timing_now();
 
 	/* wait for the execution of the tasks */
-	gettimeofday(&start_exec, NULL);
-	ret = starpu_tag_wait((starpu_tag_t)(ntasks - 1));
+	start_exec = starpu_timing_now();
+	ret = starpu_task_wait(&tasks[ntasks-1]);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_tag_wait");
-	gettimeofday(&end_exec, NULL);
+	end_exec = starpu_timing_now();
+
+	starpu_task_wait_for_all();
+
+	for (i = 0; i < ntasks; i++)
+		starpu_task_clean(&tasks[i]);
 
 	for (buffer = 0; buffer < nbuffers; buffer++)
-	{
 		starpu_data_unregister(data_handles[buffer]);
-	}
 
-	timing_submit = (double)((end_submit.tv_sec - start_submit.tv_sec)*1000000 + (end_submit.tv_usec - start_submit.tv_usec));
-	timing_exec = (double)((end_exec.tv_sec - start_exec.tv_sec)*1000000 + (end_exec.tv_usec - start_exec.tv_usec));
+	timing_submit = end_submit - start_submit;
+	timing_exec = end_exec - start_exec;
 
 	fprintf(stderr, "Total submit: %f secs\n", timing_submit/1000000);
 	fprintf(stderr, "Per task submit: %f usecs\n", timing_submit/ntasks);
@@ -210,7 +215,7 @@ int main(int argc, char **argv)
         }
 
 	starpu_shutdown();
-
+	free(tasks);
 	return EXIT_SUCCESS;
 
 enodev:
@@ -218,5 +223,6 @@ enodev:
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
 	starpu_shutdown();
+	free(tasks);
 	return STARPU_TEST_SKIPPED;
 }
